#if !defined(__SANGWUG_KERNEL_MATH_TRANSFORMATION_VECTOR_H_)
#define __SANGWUG_KERNEL_MATH_TRANSFORMATION_VECTOR_H_ 1


#include "swMathConst.h"
#include "swMathExt.h"
#include <cmath>
#include <cassert>
#include <iostream>


//----------------------------------------------------------------------------------------------
//  class swTVector3  :  3D transformation vector ( 3x1 vector )

template<typename T>
class swTVector3
{
/**@name Type Definitions */
//@{
public :
    typedef T value_type;
//@}

/**@name Private Data Members */
//@{
private :
	T x_, y_, z_;
//@}
    
/**@name Constructor, Destructor & Assignment Operator */
//@{
public :
	swTVector3(const T& tX = T(0), const T& tY = T(0), const T& tZ = T(0))
	: x_(tX), y_(tY), z_(tZ)
	{}
	explicit swTVector3(const T rhs[3])
	: x_(rhs[0]), y_(rhs[1]), z_(rhs[2])
	{}
	swTVector3(const swTVector3& rhs)
	: x_(rhs.x_), y_(rhs.y_), z_(rhs.z_)
	{}
	~swTVector3()  {}

	swTVector3& operator=(const swTVector3& rhs)
	{
	    if (this == &rhs) return *this;
	    x_ = rhs.x_;  y_ = rhs.y_;  z_ = rhs.z_;
	    return *this;
	}
//@}

/**@name Public Member Functions */
//@{
public :
	///  accessor & mutator
	T& x()  {  return x_;  }
	const T& x() const  {  return x_;  }
	T& y()  {  return y_;  }
	const T& y() const  {  return y_;  }
	T& z()  {  return z_;  }
	const T& z() const  {  return z_;  }

	///
	T& operator[](int iIndex)
	{
		switch (iIndex) {
		case 0 : return x_;
		case 1 : return y_;
		case 2 : return z_;
		default :
			assert(false);
			return iIndex < 0 ? x_ : z_;
		}
	}
	const T& operator[](int iIndex) const
	{
		switch (iIndex) {
		case 0 : return x_;
		case 1 : return y_;
		case 2 : return z_;
		default :
			assert(false);
			return iIndex < 0 ? x_ : z_;
		}
	}

	///
	bool is_zero(const T& tTol = (T)swMathConst::EPS) const
	{  return swMathExt::is_zero(x_, tTol) && swMathExt::is_zero(y_, tTol) && swMathExt::is_zero(z_, tTol);  }
	bool is_equal(const swTVector3& rhs, const T& tTol = (T)swMathConst::EPS) const
	{  return (*this - rhs).is_zero(tTol);  }
	bool is_unit(const T& tTol = (T)swMathConst::EPS) const
	{  return swMathExt::is_zero(norm() - T(1), tTol);  }
	bool is_orthogonal(const swTVector3& rhs, const T& tTol = (T)swMathConst::EPS) const
	{
		if (swMathExt::is_zero(norm(), tTol) || swMathExt::is_zero(rhs.norm(), tTol)) return false;
		return swMathExt::is_zero(dot(rhs), tTol);
	}
	bool is_parallel(const swTVector3& rhs, const T& tTol = (T)swMathConst::EPS) const
	{
		if (swMathExt::is_zero(norm(), tTol) || swMathExt::is_zero(rhs.norm(), tTol)) return false;
		swTVector3 unit1 = unit(), unit2 = rhs.unit();
		return (unit1 - unit2).is_zero(tTol) || (unit1 + unit2).is_zero(tTol);
	}

	///  comparison operator
    bool operator==(const swTVector3& rhs) const  {  return is_equal(rhs);  }
    bool operator!=(const swTVector3& rhs) const  {  return !is_equal(rhs);  }

	///
	swTVector3& operator+()  {  return *this;  }
	const swTVector3& operator+() const  {  return *this;  }
	swTVector3 operator+(const swTVector3& rhs) const
	{  return swTVector3(x_+rhs.x_, y_+rhs.y_, z_+rhs.z_);  }
	swTVector3& operator+=(const swTVector3& rhs)
	{  x_ += rhs.x_;  y_ += rhs.y_;  z_ += rhs.z_;  return *this;  }
	swTVector3 operator-() const  {  return swTVector3(-x_, -y_, -z_);  }
	swTVector3 operator-(const swTVector3& rhs) const
	{  return swTVector3(x_-rhs.x_, y_-rhs.y_, z_-rhs.z_);  }
	swTVector3& operator-=(const swTVector3& rhs)
	{  x_ -= rhs.x_;  y_ -= rhs.y_;  z_ -= rhs.z_;  return *this;  }

	///  dot product
	T operator*(const swTVector3& rhs) const  {  return dot(rhs);  }

	///  scalar operation
	swTVector3 operator*(const T& S) const
	{  return swTVector3(x_*S, y_*S, z_*S);  }
	swTVector3& operator*=(const T& S)
	{  x_ *= S;  y_ *= S;  z_ *= S;  return *this;  }
	swTVector3 operator/(const T& S) const
	{
		if (swMathExt::is_zero(S)) {
			assert(false);
			return *this;
		}
	    return swTVector3(x_/S, y_/S, z_/S);
	}
	swTVector3& operator/=(const T& S)
	{
		if (swMathExt::is_zero(S)) {
			assert(false);
			return *this;
		}
	    x_ /= S;  y_ /= S;  z_ /= S;
	    return *this;
	}

	///  Euclidean norm or L2-norm  :  V = (|V(i)|^2)^(1/2)
	T norm() const
	{  return (T)::sqrt(x_*x_ + y_*y_ + z_*z_);  }
	///  dot product
	T dot(const swTVector3& rhs) const
	{  return x_*rhs.x_ + y_*rhs.y_ + z_*rhs.z_;  }
	///  cross product
	swTVector3 cross(const swTVector3& rhs) const
	{  return swTVector3(y_*rhs.z_ - z_*rhs.y_, z_*rhs.x_ - x_*rhs.z_, x_*rhs.y_ - y_*rhs.x_);  }
	///  unit vector
	swTVector3 unit(const T& tTol = (T)swMathConst::EPS) const
	{
		T tNorm(norm());
		if (swMathExt::is_zero(tNorm, tTol)) {
			assert(false);
			return *this;
		}
		return *this / tNorm;
	}

	///
	bool normalize(const T& tTol = (T)swMathConst::EPS)
	{
		T tNorm(norm());
		if (swMathExt::is_zero(tNorm, tTol)) {
			assert(false);
			return false;
		}
		*this /= tNorm;
		return true;
	}
/*	
	///
	swTVector2<T>& Translate(const T& tX, const T& tY);
	swTVector2<T>& Rotate(const T& tAngle);
	swTVector2<T>& Scale(const T& tX, const T& tY);
	swTVector2<T>& Mirror(const T& tAngle);
*/
	///
	void to_array(T array[3])
	{  array[0] = x_;  array[1] = y_;  array[2] = z_;  }
//@}
};


//-----------------------------------------------------------------------------------------
//  3D Transformation Vector API

template<typename T>
std::istream& operator>>(std::istream& stream, swTVector3<T>& v)
{
	//  < x , y , z > means a transformation vector
	char ch;
	stream >> ch >> v.x() >> ch >> v.y() >> ch >> v.z() >> ch;
	return stream;
}

template<typename T>
std::ostream& operator<<(std::ostream& stream, const swTVector3<T>& v)
{
	//  < x , y , z > means a transformation vector
	stream << "< " << v.x() << " , " << v.y() << " , " << v.z() << " >";
	return stream;
}


#endif  //  __SANGWUG_KERNEL_MATH_TRANSFORMATION_VECTOR_H_
